#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import os
import unittest

from mock import MagicMock
from opsramp.api import ORapi


class StaticsTest(unittest.TestCase):
    def check64(self, testfile):
        with open(testfile, "rb") as f:
            content_raw = f.read()
        content_64 = ORapi.b64encode_payload(testfile)
        # decode this result and check against the actual content.
        decoded = base64.b64decode(content_64)
        return decoded == content_raw

    def test_base64(self):
        # a file that is guaranteed to be empty.
        assert self.check64(os.devnull)
        # any non-empty file that we know will exist.
        assert self.check64("setup.py")


class ClassTest(unittest.TestCase):
    def setUp(self):
        self.mock_ao = MagicMock()
        self.mock_ao.clone.return_value = self.mock_ao
        self.testobj = ORapi(self.mock_ao)

    def test_str(self):
        assert "ORapi" in str(self.testobj)

    def test_session_property(self):
        # The session property is defined in the base class but we
        # will unit test it here anyway, to make sure nobody changes
        # the base class in a way that breaks us.
        aw = self.testobj
        original_session = aw.session
        assert original_session
        # test the setter and getter
        fake_session = MagicMock()
        aw.session = fake_session
        assert aw.session is fake_session
        # put the real one back
        aw.session = original_session
        assert aw.session is original_session

    def test_get(self):
        hdrs = {"fake-header": "fake-value"}
        expected = "unit test get result"
        self.mock_ao.get.return_value = expected

        suffix = "milk"
        actual = self.testobj.get(suffix, headers=hdrs)
        self.mock_ao.get.assert_called_with(suffix, headers=hdrs)
        assert actual == expected

    def test_search(self):
        hdrs = {"fake-header": "fake-value"}
        expected = "unit test search result"
        self.mock_ao.get.return_value = expected

        # the leading ? character is optional so try both.
        qstring = "?name=marmaduke"
        qs2 = qstring[1:]

        # default suffix should be "search"
        actual = self.testobj.search(pattern=qstring)
        self.mock_ao.get.assert_called_with("search" + qstring, headers=None)
        assert actual == expected
        actual = self.testobj.search(pattern=qs2, headers=hdrs)
        self.mock_ao.get.assert_called_with("search" + qstring, headers=hdrs)
        assert actual == expected

        # try a different suffix
        suffix = "toraiocht"
        actual = self.testobj.search(pattern=qstring, suffix=suffix)
        self.mock_ao.get.assert_called_with(suffix + qstring, headers=None)
        assert actual == expected
        actual = self.testobj.search(pattern=qs2, headers=hdrs, suffix=suffix)
        self.mock_ao.get.assert_called_with(suffix + qstring, headers=hdrs)
        assert actual == expected



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# kb.py
# Classes dealing directly with OpsRamp knowledge base categories
# and articles.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class KnowledgeBase(ORapi):
    def __init__(self, parent):
        super(KnowledgeBase, self).__init__(parent.api, "kb")

    def categories(self):
        return KBcategories(self)

    def articles(self):
        return KBarticles(self)

    def templates(self):
        return KBtemplates(self)


class KBcategories(ORapi):
    def __init__(self, parent):
        super(KBcategories, self).__init__(parent.api, "category")
        # weirdly, calls to "categorylist" need to be made at "kb" level
        # so we need to keep a handle on the parent object's API.
        self.parent_api = parent.api.clone()

    def create(self, definition):
        resp = self.api.post("create", json=definition)
        return resp

    def update(self, uuid, definition):
        return self.api.post("update/%s" % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete("delete/%s" % uuid)

    def search(self, pattern=""):
        suffix = "categorylist"
        if pattern:
            suffix += "?" + pattern
        # weirdly, this call needs to be made at "kb" level
        return self.parent_api.get(suffix)

    def children(self, uuid):
        assert uuid
        suffix = "categorylist/%s" % uuid
        # weirdly, this call needs to be made at "kb" level
        return self.parent_api.get(suffix)

    def restore(self, uuid):
        return self.api.post("restore/%s" % uuid)


class KBarticles(ORapi):
    def __init__(self, parent):
        super(KBarticles, self).__init__(parent.api, "article")
        # weirdly, calls to "articlesList" need to be made at "kb" level
        # so we need to keep a handle on the parent object's API.
        self.parent_api = parent.api.clone()

    def create(self, definition):
        resp = self.api.post(json=definition)
        return resp

    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete("%s/delete" % uuid)

    def search(self, pattern=""):
        suffix = "articles/search"
        if pattern:
            suffix += "?" + pattern
        # weirdly, this call needs to be made at "kb" level
        return self.parent_api.get(suffix)

    def share(self, uuid, definition):
        return self.api.post("%s/share" % uuid, json=definition)

    def comments(self, uuid):
        return self.api.get("%s/comments" % uuid)


class KBtemplates(ORapi):
    def __init__(self, parent):
        super(KBtemplates, self).__init__(parent.api, "template")
        # weirdly, calls to "templatesList" need to be made at "kb" level
        # so we need to keep a handle on the parent object's API.
        self.parent_api = parent.api.clone()

    def create(self, definition):
        resp = self.api.post(json=definition)
        return resp

    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete("%s/delete" % uuid)

    def search(self, pattern=""):
        suffix = "templatesList"
        if pattern:
            suffix += "?" + pattern
        # this call needs to be made at "kb" level
        return self.parent_api.get(suffix)



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

    def test_get(self):
        group = self.client.resource_groups()
        fake_raw_result = ["unit", "test", "get", "result"]
        with requests_mock.Mocker() as m:
            url_suffix = "/search"
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result)
            actual = group.get()
            assert actual == fake_raw_result

    def test_create(self):
        group = self.client.resource_groups()
        fake_create_json = [
            {
                "resourceName": "ABBY-PC",
                "hostName": "ABBY-PC",
                "aliasName": "Server PC",
                "resourceType": "server",
                "os": "Ubuntu 14.04.6 LTS",
                "serialNumber": "1234-5678-901234",
            }
        ]
        thisid = 333333
        fake_result = [{"id": thisid}]
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.create(definition=fake_create_json)
            assert actual == fake_result

    def test_update(self):
        group = self.client.resource_groups()
        fake_update_json = [{"id": "123456", "os": "Ubuntu 18.04.4 LTS"}]
        fake_result = fake_update_json
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.update(definition=fake_update_json)
            assert actual == fake_result

    def test_delete(self):
        group = self.client.resource_groups()
        fake_resource_id = "789012"
        fake_result = {"id": fake_resource_id}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(fake_resource_id)
            m.delete(url, json=fake_result, complete_qs=True)
            actual = group.delete(uuid=fake_resource_id)
            assert actual == fake_result

    def test_search(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resource_groups()
        fake_search_pattern = "queryString=agentInstalled:true"
        fake_raw_result = ["unit", "test", "search", "values"]
        count = len(fake_raw_result)
        fake_cooked_result = {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": fake_raw_result,
        }
        with requests_mock.Mocker() as m:
            url_suffix = "search?{0}".format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.search(pattern=fake_search_pattern)
            assert actual == fake_cooked_result

    def test_minimal(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resource_groups()
        fake_search_pattern = "queryString=agentInstalled:true"
        fake_raw_result = ["unit", "test", "minimal", "result"]
        count = len(fake_raw_result)
        fake_cooked_result = {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": fake_raw_result,
        }
        with requests_mock.Mocker() as m:
            url_suffix = "minimal?{0}".format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.minimal(pattern=fake_search_pattern)
            assert actual == fake_cooked_result



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from mock import MagicMock
from opsramp.base import ApiWrapper


class ClassTest(unittest.TestCase):
    def setUp(self):
        self.mock_ao = MagicMock()
        self.mock_ao.clone.return_value = self.mock_ao
        self.testobj = ApiWrapper(self.mock_ao)

    def test_str(self):
        assert "ApiWrapper" in str(self.testobj)

    def test_session_property(self):
        aw = self.testobj
        original_session = aw.session
        assert original_session
        # test the setter and getter
        fake_session = MagicMock()
        aw.session = fake_session
        assert aw.session is fake_session
        # put the real one back
        aw.session = original_session
        assert aw.session is original_session

    def test_get(self):
        ut_hdrs = {"fake-header": "fake-value"}
        expected = "unit test get result"
        self.mock_ao.get.return_value = expected

        suffix = "milk"
        # default headers
        actual = self.testobj.get(suffix)
        self.mock_ao.get.assert_called_with(suffix, headers=None)
        assert actual == expected
        # specific headers
        actual = self.testobj.get(suffix, headers=ut_hdrs)
        self.mock_ao.get.assert_called_with(suffix, headers=ut_hdrs)
        assert actual == expected

    def four_variants(self, suffix, test_fn, mock_ao_fn):
        ut_hdrs = {"fake-header": "fake-value"}
        ut_text = "some unit test string"
        ut_json = {"id": "unit test"}
        expected = "unit test result"
        mock_ao_fn.return_value = expected

        # all defaults
        actual = test_fn()
        mock_ao_fn.assert_called_with(None, headers=None, data=None, json=None)

        # with suffix
        actual = test_fn(suffix)
        mock_ao_fn.assert_called_with(suffix, headers=None, data=None, json=None)
        # specific headers
        actual = test_fn(suffix, headers=ut_hdrs)
        mock_ao_fn.assert_called_with(suffix, headers=ut_hdrs, data=None, json=None)
        assert actual == expected
        # specific text body
        actual = test_fn(suffix, data=ut_text)
        mock_ao_fn.assert_called_with(suffix, headers=None, data=ut_text, json=None)
        assert actual == expected
        # specific json body
        actual = test_fn(suffix, json=ut_json)
        mock_ao_fn.assert_called_with(suffix, headers=None, data=None, json=ut_json)
        assert actual == expected

    def five_variants(self, suffix, test_fn, mock_ao_fn):
        ut_hdrs = {"fake-header": "fake-value"}
        ut_text = "some unit test string"
        ut_json = {"id": "unit test"}
        ut_file = {"attachment": "testing.csv"}
        expected = "unit test result"
        mock_ao_fn.return_value = expected

        # all defaults
        actual = test_fn()
        mock_ao_fn.assert_called_with(
            None, headers=None, data=None, json=None, files=None
        )

        # with suffix
        actual = test_fn(suffix)
        mock_ao_fn.assert_called_with(
            suffix, headers=None, data=None, json=None, files=None
        )
        # specific headers
        actual = test_fn(suffix, headers=ut_hdrs)
        mock_ao_fn.assert_called_with(
            suffix, headers=ut_hdrs, data=None, json=None, files=None
        )
        assert actual == expected
        # specific text body
        actual = test_fn(suffix, data=ut_text)
        mock_ao_fn.assert_called_with(
            suffix, headers=None, data=ut_text, json=None, files=None
        )
        assert actual == expected
        # specific json body
        actual = test_fn(suffix, json=ut_json)
        mock_ao_fn.assert_called_with(
            suffix, headers=None, data=None, json=ut_json, files=None
        )
        assert actual == expected
        actual = test_fn(suffix, files=ut_file)
        mock_ao_fn.assert_called_with(
            suffix, headers=None, data=None, json=None, files=ut_file
        )
        assert actual == expected

    def test_post(self):
        self.five_variants("eggs", self.testobj.post, self.mock_ao.post)

    def test_put(self):
        self.four_variants("flour", self.testobj.put, self.mock_ao.put)

    def test_delete(self):
        self.four_variants("sultanas", self.testobj.delete, self.mock_ao.delete)

    def test_patch(self):
        self.four_variants("meat", self.testobj.patch, self.mock_ao.patch)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# tenant.py
# Classes dealing directly with OpsRamp Tenants.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from opsramp.api import ORapi
import opsramp.devmgmt
import opsramp.escalations
import opsramp.first_response
import opsramp.integrations
import opsramp.kb
import opsramp.mgmt_profiles
import opsramp.monitoring
import opsramp.msp
import opsramp.rba
import opsramp.resource_groups
import opsramp.resources
import opsramp.roles
import opsramp.service_maps
import opsramp.sites


class Tenant(ORapi):
    def __init__(self, parent, uuid):
        super(Tenant, self).__init__(parent.api, "tenants/%s" % uuid)
        self.uuid = uuid

    def is_client(self):
        return self.uuid[:7] == "client_"

    def rba(self):
        return opsramp.rba.Rba(self)

    def monitoring(self):
        return opsramp.monitoring.Monitoring(self)

    def clients(self):
        assert not self.is_client()
        return opsramp.msp.Clients(self)

    def policies(self):
        return opsramp.devmgmt.Policies(self)

    def discovery(self):
        return opsramp.devmgmt.Discovery(self)

    def integrations(self):
        return opsramp.integrations.Integrations(self)

    def get_agent_script(self):
        assert self.is_client()
        hdr = {"Accept": "application/octet-stream,application/xml"}
        return self.api.get("agents/deployAgentsScript", headers=hdr)

    def credential_sets(self):
        return opsramp.devmgmt.CredentialSets(self)

    def resources(self):
        return opsramp.resources.Resources(self)

    def roles(self):
        return opsramp.roles.Roles(self)

    def permission_sets(self):
        return opsramp.roles.PermissionSets(self)

    def escalations(self):
        return opsramp.escalations.Escalations(self)

    def mgmt_profiles(self):
        return opsramp.mgmt_profiles.Profiles(self)

    def sites(self):
        return opsramp.sites.Sites(self)

    def service_maps(self):
        return opsramp.service_maps.ServiceMaps(self)

    def kb(self):
        return opsramp.kb.KnowledgeBase(self)

    def first_response(self):
        return opsramp.first_response.First_Response(self)

    def model_training(self):
        return opsramp.first_response.ModelTraining(self)

    def resource_groups(self):
        return opsramp.resource_groups.ResourceGroups(self)



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from contextlib import contextmanager
from copy import deepcopy
from http.server import BaseHTTPRequestHandler, HTTPServer
from io import StringIO
import logging
import socket
import sys
from threading import Thread
import unittest

from opsramp.base import ApiObject, ApiWrapper
import requests

# Define a list of "canned" responses here. These are used to test the retry
# capability of the API client, so that when faced with a response containing a
# HTTP 429 (Too Many Requests) status code, it should keep retrying until some
# maximum limit is reached.
CANNED_RESPONSES = [
    {"code": 429, "message": b"Failed attempt #1"},
    {"code": 429, "message": b"Failed attempt #2"},
    {"code": 429, "message": b"Failed attempt #3"},
    {"code": 200, "message": b"Succeeded on fourth attempt"},
]


# Borrowed from https://gist.github.com/mogproject/fc7c4e94ba505e95fa03
@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err


class MockServerRequestHandler(BaseHTTPRequestHandler, object):
    def __init__(self, request, client_address, server):
        # Creating this reference to the server as that is where the current
        # server instance's copies of the canned responses live; a new request
        # handler is generated for each request so they can't live here.
        self.http_server = server
        super(MockServerRequestHandler, self).__init__(request, client_address, server)

    def do_GET(self):
        logging.debug("running do_GET()")

        if len(self.http_server.canned_responses) > 0:
            # We have at least one more "canned" response to send...
            response = self.http_server.canned_responses.pop(0)
            logging.debug("Sending response: %s" % response)

            # Send the header including the status code
            self.send_response(code=response["code"])
            self.send_header("Content-type", "text/plain")
            self.end_headers()

            # Write the message body to self.wfile
            self.wfile.write(response["message"])

            # self.canned_responses = self.canned_responses[1:]
            logging.debug(
                "%d responses remain" % len(self.http_server.canned_responses)
            )
        else:
            # If we run out of canned responses just send back a HTTP 500
            # (Internal Server Error). The idea is that the retry logic should
            # give up before the test server runs out of canned responses and
            # that this is here just for completeness.
            self.send_response(code=500)

            self.send_header("Content-type", "text/plain")
            self.end_headers()

            self.wfile.write(b"Internal Server Error")

        return


class MockHttpServer(HTTPServer, object):
    def __init__(self, server_address, RequestHandlerClass, canned_responses):
        # Need to subclass HTTPServer so that I can define a set of canned
        # responses that will be consumed by MockServerRequestHandler.
        logging.warning("Standing up HTTP Server")
        self.canned_responses = canned_responses
        super(MockHttpServer, self).__init__(server_address, RequestHandlerClass)


class MockWebClient:
    """Minimal web client to generate requests against our mock web server"""

    def __init__(self, host="localhost", port=80):
        self.host = host
        self.port = port

    def get(self):
        return requests.get("http://%s:%d/test" % (self.host, self.port))


class TestMockServer(unittest.TestCase):
    def setUp(self):
        self.server_port = self.get_free_port()
        logging.info("Creating HTTP server on port %d" % self.server_port)
        self.mock_server = MockHttpServer(
            ("localhost", self.server_port),
            MockServerRequestHandler,
            deepcopy(CANNED_RESPONSES),
        )

        # Create the server in a separate thread so that it can run in parallel
        # with the client (i.e. tests) being run against it.
        self.mock_server_thread = Thread(target=self.mock_server.serve_forever)
        self.mock_server_thread.setDaemon(True)
        self.mock_server_thread.start()

    def tearDown(self):
        logging.info("Shutting down Mock HTTP server")
        self.mock_server.shutdown()
        return super(TestMockServer, self).tearDown()

    @classmethod
    def get_free_port(cls):
        """Get the system to open a TCP socket (and let it choose the port
        number). Once it's been created successfully, shut down the empty
        server attached to that socket and return the port number to be
        "recycled" for use with the test server.

        Returns:
            int -- Available TCP Port number
        """
        s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
        s.bind(("localhost", 0))
        _, port = s.getsockname()
        s.close()
        return port

    def test__mock_server_works(self):
        # Test that the web server returns all canned responses, in order.
        client = MockWebClient(host="localhost", port=self.server_port)

        for expected in CANNED_RESPONSES:
            response = client.get()

            self.assertEqual(response.status_code, expected["code"])
            self.assertEqual(response.content, expected["message"])

    def test__apiwrapper_retry_logic(self):
        fake_url = "http://localhost:%d/" % self.server_port
        fake_token = "ffffffffffffffff"
        fake_auth = {
            "Authorization": "Bearer %s" % fake_token,
            "Accept": "application/json",
        }

        api_object = ApiObject(fake_url, fake_auth.copy())
        assert "ApiObject" in str(api_object)

        api_stub = "whatevs"
        api_wrapper = ApiWrapper(api_object, api_stub)

        api_endpoint = "foo"
        with captured_output() as (out, err):
            api_wrapper.api.get("/%s" % api_endpoint)

        expected_requests = []
        for resp in CANNED_RESPONSES:
            req = '"GET /%s/%s HTTP/1.1" %d' % (api_stub, api_endpoint, resp["code"])
            expected_requests.append(req)

        lines = err.getvalue().splitlines()

        # Check each GET request to verify that it gets the correct status code
        # (basically, 429 three times then a 200) to confirm that the retry
        # logic is doing its thing
        for index in range(len(lines)):
            self.assertIn(expected_requests[index], lines[index])



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.first_response = self.client.first_response()
        assert "First_Response" in str(self.first_response)

        self.model_training = self.client.model_training()
        assert "ModelTraining" in str(self.model_training)

    def test_search(self):
        group = self.first_response
        for pattern, expected in (
            ("", ["unit", "test", "results"]),
            ("pageNo=1&pageSize=100&is&sortName=id", ["more", "nonsense"]),
        ):
            if pattern:
                url = group.api.compute_url("?" + pattern)
            else:
                url = group.api.compute_url()
            with requests_mock.Mocker() as m:
                m.get(url, json=expected, complete_qs=True)
                actual = group.search(pattern)
            assert actual == expected

    def test_policy_detail(self):
        group = self.first_response
        thisid = 789012
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.policy_detail(uuid=thisid)
            assert actual == expected

    def test_create(self):
        group = self.first_response
        expected = {"id": 345678}
        fake_defn = {"name": "dougal"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=fake_defn)
            assert actual == expected

    def test_update(self):
        group = self.first_response
        thisid = 123456
        expected = {"id": thisid}
        fake_defn = {"name": "ted"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_delete(self):
        group = self.first_response
        thisid = 789012
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected

    def test_enable(self):
        group = self.first_response
        thisid = 345678
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/enable" % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.enable(uuid=thisid)
            assert actual == expected

    def test_disable(self):
        group = self.first_response
        thisid = 901234
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/disable" % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.disable(uuid=thisid)
            assert actual == expected

    def test_model_training(self):
        group = self.model_training
        expected = "sample text"
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("train/ALERT_FIRST_RESPONSE_TRAINING")
            m.post(url, text=expected, complete_qs=True)
            actual = group.train_model()
            assert actual == expected

    def test_file_upload(self):
        group = self.model_training
        expected = "sample text"
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("files")
            with open("tests/testing.csv", "rb") as x:
                file = {"attachment": ("testing.csv", x, "text/csv")}
                data = {"test": "fake"}
                m.post(url, text=expected, complete_qs=True)
                actual = group.file_upload(data, file)
                assert actual == expected

    def test_get_training_file(self):
        group = self.model_training
        expected = {
            "results": [{"name": "sample"}],
            "totalResults": 1,
            "orderBy": "createdTime",
            "pageNo": 1,
            "pageSize": 100,
            "totalPages": 1,
            "nextPage": False,
            "previousPageNo": 0,
            "descendingOrder": True,
        }
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("files")
            m.get(url, json=expected, complete_qs=True)
            actual = group.get_training_file()
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class BindingTest(unittest.TestCase):
    def setUp(self):
        endpoint = "mock://api.example.com"
        key = "opensesame"
        secret = "thereisnospoon"
        token = "fake-unit-test-token"

        url = endpoint + "/auth/oauth/token"
        hshake = "grant_type=client_credentials&client_id=%s&client_secret=%s"
        expected_send = hshake % (key, secret)
        expected_receive = {"access_token": token}
        expected_auth = {
            "Authorization": "Bearer " + token,
            "Accept": "application/json,application/xml",
        }
        with requests_mock.Mocker() as m:
            adapter = m.post(url, json=expected_receive, complete_qs=True)
            self.ormp = opsramp.binding.connect(endpoint, key, secret)
            assert m.call_count == 1
            assert adapter.last_request.text == expected_send
            assert type(self.ormp) is opsramp.binding.Opsramp
            assert self.ormp.auth == expected_auth

    def test_str(self):
        assert "Opsramp" in str(self.ormp)

    def test_config(self):
        obj = self.ormp.config()
        assert "GlobalConfig" in str(obj)

    def test_tenant(self):
        obj = self.ormp.tenant("unit-test")
        assert "Tenant" in str(obj)



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding
import yaml


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ["OPSRAMP_TENANT_ID"]

    ormp = connect()
    tenant = ormp.tenant(tenant_id)

    jdata = {
        "name": "Policy_test",
        "filterCriteria": {
            "filterBased": "true",
            "matchingType": "ALL",
            "rules": [
                {
                    "filterType": "native",
                    "entityName": "host_name",
                    "operator": "Starts With",
                    "entityValue": "value_of_attribute",
                }
            ],
        },
        "firstResponseType": "SUPPRESSION",
        "suppression": {
            "suppressSeasonalAlerts": "true",
            "suppressByAttributes": "true",
            "autoSnooze": "true",
            "continuousLearning": "true",
            "trainingFileId": "ml_alert_first_response_training",
        },
    }
    frpolicies = tenant.first_response()
    resp = frpolicies.create(jdata)
    print(yaml.dump(resp, default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class TenantTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(fake_client_id)
        assert self.client.is_client()

        fake_msp_id = "msp_for_unit_test"
        self.msp = self.ormp.tenant(fake_msp_id)
        assert not self.msp.is_client()

    def test_agent_script(self):
        with requests_mock.Mocker() as m:
            url = self.client.api.compute_url("agents/deployAgentsScript")
            expected = "unit test fake agent content"
            m.get(url, text=expected, complete_qs=True)
            actual = self.client.get_agent_script()
            assert actual == expected
            assert m.call_count == 1

    def test_clients(self):
        # A partner object can have clients contained within it.
        assert not self.msp.is_client()
        assert self.msp.clients()
        # A client object cannot have other clients nested inside it
        # so it should raise an assertion if we try to access them.
        assert self.client.is_client()
        with self.assertRaises(AssertionError):
            assert self.client.clients()

    def test_methods_exist(self):
        assert self.client.rba()
        assert self.client.monitoring()
        assert self.client.policies()
        assert self.client.discovery()
        assert self.client.integrations()
        assert self.client.credential_sets()
        assert self.client.roles()



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding
import opsramp.integrations


subscription = os.environ["AZURE_SUBSCRIPTION"]
azure_tenant = os.environ["AZURE_TENANT_ID"]
azure_client = os.environ["AZURE_CLIENT_ID"]
azure_secret = os.environ["AZURE_SECRET_KEY"]


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ["OPSRAMP_TENANT_ID"]

    ormp = connect()
    tnt = ormp.tenant(tenant_id)
    discovery = tnt.discovery()

    # Create new discovery profile...
    creds = opsramp.Integrations.mkAzureARM(
        arm_subscription_id=subscription,
        arm_tenant_id=azure_tenant,
        arm_client_id=azure_client,
        arm_secret_key=azure_secret,
    )
    jdata = [
        {
            "name": "Azure " + subscription,
            "credential": creds,
            "policy": {
                "name": "whatever",
                "resourceType": "ALL",
                "entityType": "ALL",
                "rules": [{"filterType": "ANY_CLOUD_RESOURCE"}],
                "actions": [
                    {
                        "action": "MANAGE DEVICE",
                        "items": [],
                        "forceAssignOrUnassign": False,
                    }
                ],
                "matchType": "ANY",
            },
            "schedule": {
                "patternType": "MINUTES",
                "pattern": "30",
                "startTime": "00:30:00",
            },
        }
    ]

    resp = discovery.create(jdata)
    print(resp)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# service_maps.py
# Classes dealing directly with OpsRamp Service Maps.
# Service Maps are used to organise resources by any method and
# create availability rules governing how they show problems.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class ServiceMaps(ORapi):
    def __init__(self, parent):
        super(ServiceMaps, self).__init__(parent.api, "serviceGroups")

    def get(self, uuid=None, minimal=False):
        if uuid:
            temp_url = "/%s/childs/search" % uuid
        else:
            temp_url = "/search"
        if minimal:
            temp_url = "/minimal"
        return self.api.get(temp_url)

    def create(self, definition):
        return self.api.post("", json=definition)

    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete("%s" % uuid)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# devmgmt.py
# Device management classes.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class Policies(ORapi):
    def __init__(self, parent):
        super(Policies, self).__init__(parent.api, "policies/management")

    def search(self, policy_name=""):
        """For historical reasons this class's search function might be
        called with just a name. We have to cope and convert it into a
        proper query string."""
        if policy_name and "=" not in policy_name:
            qstring = "name=" + policy_name
        else:
            qstring = policy_name
        # we have assembled a proper query string now so use regular search.
        return super(Policies, self).search(pattern=qstring)

    def create(self, definition):
        return self.api.post("", json=definition)

    def update(self, uuid, definition):
        return self.api.put("%s" % uuid, json=definition)

    def run(self, uuid):
        return self.api.get("%s/action/run" % uuid)

    def delete(self, uuid):
        return self.api.delete("%s" % uuid)


class Discovery(ORapi):
    def __init__(self, parent):
        assert parent.is_client()
        super(Discovery, self).__init__(parent.api, "policies/discovery")

    def search(self, profile_name=""):
        """For historical reasons this class's search function might be
        called with just a name. We have to cope and convert it into a
        proper query string."""
        if profile_name and "=" not in profile_name:
            qstring = "name=" + profile_name
        else:
            qstring = profile_name
        # we have assembled a proper query string now so use regular search.
        return super(Discovery, self).search(pattern=qstring)

    def create(self, definition):
        return self.api.post("", json=definition)

    def update(self, definition):
        return self.api.post("", json=definition)

    def rescan(self, uuid):
        return self.api.get("/action/scan/%s" % uuid)

    def delete(self, uuid):
        return self.api.delete("/%s" % uuid)


class CredentialSets(ORapi):
    def __init__(self, parent):
        super(CredentialSets, self).__init__(parent.api, "credentialSets")

    def get(self, uuid="", minimal=False):
        if minimal:
            return self.api.get("/%s/minimal" % uuid)
        else:
            return self.api.get("/%s" % uuid)

    def create(self, definition):
        return self.api.post("", json=definition)

    def update(self, uuid, definition):
        return self.api.post("/%s" % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete("/%s" % uuid)



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    parser.add_argument("uuid", type=str)
    ns = parser.parse_args()
    return ns


def create_command_script(targetcat):
    # Create a new RBA script in this category, with one parameter.
    # OpsRamp will prompt for a value for this parameter each time
    # you run this script in the UI and will pass the value in as $1
    # which you can see being used in the "payload" below.
    p1 = targetcat.mkParameter(
        name="venue", description="Where am I today?", datatype="STRING"
    )
    s1 = targetcat.mkScript(
        name="Hello <venue>",
        description="Stereotypical rock star intro",
        platforms=["LINUX"],
        execution_type="COMMAND",
        payload='echo "hello $1"',
        parameters=[p1],
    )
    print("creating new COMMAND script...")
    print(s1)
    resp = targetcat.create(s1)
    return resp


def create_python_script(targetcat):
    scriptfile = "/tmp/whatever-%s.py" % os.getpid()
    with open(scriptfile, "w") as f:
        f.write(
            """#! /usr/bin/env python
from __future__ import print_function
print('hello world')
"""
        )
    s1 = targetcat.mkScript(
        name="Python hello",
        description="Hello from Python land",
        platforms=["LINUX"],
        execution_type="PYTHON",
        script_name=os.path.basename(scriptfile),
        payload_file=scriptfile,
    )
    print("creating new PYTHON script...")
    print(s1)
    resp = targetcat.create(s1)
    os.remove(scriptfile)
    return resp


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
    category_id = int(ns.uuid)

    tenant_id = os.environ["OPSRAMP_TENANT_ID"]

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    targetcat = tenant.rba().categories().category(category_id)

    # A "COMMAND" script is a basic oneliner that doesn't require a shell.
    resp = create_command_script(targetcat)
    print(resp)

    resp = create_python_script(targetcat)
    print(resp)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A command line interface to OpsRamp that illustrates how to use
# this language binding as well as being useful in its own right.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import os

import opsramp.binding


# TODO make these optional command line parameters
OPSRAMP_URL = os.environ["OPSRAMP_URL"]
OPSRAMP_KEY = os.environ["OPSRAMP_KEY"]
OPSRAMP_SECRET = os.environ["OPSRAMP_SECRET"]
OPSRAMP_TENANT_ID = os.environ["OPSRAMP_TENANT_ID"]


def do_auth():
    return opsramp.binding.connect(OPSRAMP_URL, OPSRAMP_KEY, OPSRAMP_SECRET)


# Ideas for the cli syntax:
#
# ormpcli \
#   --tenant=client_2703 \
#   --endpoint=https://hpe-dev.api.try.opsramp.com \
#   --key='redacted' \
#   --secret='redacted' \
#   subtree area action
#
# ormpcli tenant agent script
# ormpcli tenant rba categories
# ormpcli tenant monitoring templates


def parse_args():
    parser = argparse.ArgumentParser(description="HPE cli for OpsRamp")
    parser.add_argument("subtree")
    parser.add_argument("area")
    parser.add_argument("action")
    return parser.parse_args()


def do_tenant_rba_action(rba, action):
    if action == "categories":
        clist = rba.categories().get()
        print(json.dumps(clist))
    else:
        raise ValueError(rba, action)


def do_tenant_monitoring_action(monitoring, action):
    if action == "templates":
        tplates = monitoring.templates()
        resp = tplates.search()
        print(resp["totalResults"], "monitoring templates found")
    else:
        raise ValueError(monitoring, action)


def do_tenant_action(tenant, area, action):
    if area == "rba":
        subapi = tenant.rba()
        return do_tenant_rba_action(subapi, action)
    elif area == "monitoring":
        subapi = tenant.monitoring()
        return do_tenant_monitoring_action(subapi, action)
    elif area == "agent" and action == "script":
        contents = tenant.get_agent_script()
        print(contents)
    else:
        raise ValueError(tenant, area, action)


def do_action(ormp, subtree, area, action):
    if subtree == "tenant":
        tenant = ormp.tenant(OPSRAMP_TENANT_ID)
        return do_tenant_action(tenant, area, action)
    else:
        raise ValueError(subtree, area, action)


def main():
    args = parse_args()
    ormp = do_auth()
    do_action(ormp, args.subtree, args.area, args.action)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.service_maps = self.client.service_maps()
        assert "serviceGroups" in str(self.service_maps)

    def test_service_maps_get_root(self):
        group = self.service_maps
        thisid = 111111
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("search")
            m.get(url, json=expected, complete_qs=True)
            actual = group.get()
            assert actual == expected

    def test_service_maps_get_child(self):
        group = self.service_maps
        thisid = 222222
        expected = {"id": thisid}
        sgid = "some-sgid"
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/childs/search" % sgid)
            m.get(url, json=expected, complete_qs=True)
            # test uuid parameter specifically.
            actual = group.get(uuid=sgid)
            assert actual == expected
            # test that uuid is the first parameter
            actual = group.get(sgid)
            assert actual == expected

    def test_service_maps_get_minimal(self):
        group = self.service_maps
        thisid = 333333
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("minimal")
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(minimal=True)
            assert actual == expected

    def test_service_maps_create(self):
        group = self.service_maps
        thisid = 444444
        expected = {"id": thisid}
        fake_defn = {"name": "fr dick byrne"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=fake_defn)
            assert actual == expected

    def test_service_maps_update(self):
        group = self.service_maps
        thisid = 123456
        expected = {"id": thisid}
        fake_defn = {"name": "fr todd unctious"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_service_maps_delete(self):
        group = self.service_maps
        thisid = 789012
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class GlobalConfigTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)
        self.gconfig = self.ormp.config()
        assert "GlobalConfig" in str(self.gconfig)

    def test_simple_urls(self):
        fnmap = {
            "/alertTypes": self.gconfig.get_alert_types,
            "/cfg/countries": self.gconfig.get_countries,
            "/cfg/timezones": self.gconfig.get_timezones,
            "/cfg/alertTechnologies": self.gconfig.get_alert_technologies,
            "/cfg/tenants/channels": self.gconfig.get_channels,
            "/cfg/devices/types": self.gconfig.get_device_types,
        }
        for suffix, fn in fnmap.items():
            with requests_mock.Mocker() as m:
                url = self.ormp.api.compute_url(suffix)
                # all these methods return lists so create a nonsense one.
                expected = ["unit", "test"]
                expected.extend(suffix.split("/"))
                assert expected
                m.get(url, json=expected, complete_qs=True)
                actual = fn()
                assert actual == expected

    # Test the bizarre special cases around NOT FOUND that are
    # documented in the definitiom of get_nocs.
    def test_irregular_urls(self):
        fnmap = {"/cfg/tenants/nocs": self.gconfig.get_nocs}
        for suffix, fn in fnmap.items():
            with requests_mock.Mocker() as m:
                # rig the mock to raise an exception on "get"
                url = self.ormp.api.compute_url(suffix)
                expected = "nonsense unit test value"
                for eclass in (RuntimeError, AssertionError, Exception):
                    m.get(url, exc=eclass("omg the sky is falling"))
                    with self.assertRaises(eclass):
                        actual = fn()
                # now try the special case exception that fn() *should* handle.
                expected = []
                m.get(url, exc=RuntimeError('"code":"0005" rien de rien'))
                actual = fn()
                assert actual == expected



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# base.py
# Containing various base classes used in other parts of the library
# but not intended for direct use by callers.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from urllib import parse as urlparse

import requests
from simplejson.errors import JSONDecodeError


LOG = logging.getLogger(__name__)


class Helpers(object):
    # (DW) Add support for retries of requests to the OpsRamp API in the event
    # of receiving a HTTP 429 (Too Many Requests) response from the API to
    # suggest that it has activated rate limiting. This implements progressive
    # backoff (i.e. gradually increasing the delay between attempts) until the
    # maximum number of retries is reached.
    # By wrapping this retry handler around the session instance being used
    # inside instances of the ApiWrapper class the effect of this should be
    # more or less transparent to the rest of the code; at least until rate
    # limiting kicks in, at which point it will hopefully slow down but still
    # "get there" so to speak, and seems to be non-"hacky".
    # The defaults *should* be sensible.
    # Note: if the status_forcelist tuple only has one value (e.g 429) then a
    # trailing comma is REQUIRED for Python to interpret it correctly as a
    # tuple: type((429)) is 'int', whereas type((429,)) is 'tuple'.
    # Borrowed from:
    # https://www.peterbe.com/plog/best-practice-with-retries-with-requests

    retryclass = requests.packages.urllib3.util.Retry

    @staticmethod
    def default_retry_verbs() -> frozenset:
        try:
            retval = Helpers.retryclass.DEFAULT_ALLOWED_METHODS
        except AttributeError:
            # try the deprecated name from older versions of urllib3
            retval = Helpers.retryclass.DEFAULT_METHOD_WHITELIST
        return retval

    @staticmethod
    def session_add_retry_handler(session=None):
        # urllib3 does not retry on POST by default, but we want to iff the
        # return status is 429 rate limiting, on the assumption that this
        # means the POST did not happen and is therefore safe to retry.
        http_verbs = set(Helpers.default_retry_verbs())
        http_verbs.add("POST")
        retry = Helpers.create_retry_handler(
            retries=7,
            backoff_factor=0.5,
            status_forcelist=(429,),
            allowed_methods=http_verbs,
        )

        adapter = requests.adapters.HTTPAdapter(max_retries=retry)

        session = session or requests.Session()
        session.mount(prefix="http://", adapter=adapter)
        session.mount(prefix="https://", adapter=adapter)
        return session

    @staticmethod
    def create_retry_handler(
        retries, backoff_factor, status_forcelist, allowed_methods
    ):
        assert isinstance(retries, int)
        assert retries >= 0
        assert backoff_factor >= 0
        assert isinstance(status_forcelist, tuple)

        return Helpers.retryclass(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
        )


class PathTracker(object):
    def __init__(self):
        self.reset()

    def reset(self):
        self.prefix = ""
        self.stack = []

    def __str__(self):
        return '%s "%s" %s' % (str(type(self)), self.prefix, self.stack)

    def clone(self):
        new1 = PathTracker()
        new1.prefix = self.prefix
        new1.stack = self.stack
        return new1

    def cd(self, path=None):
        path = str(path or "/")
        # no support for '..' right now, maybe in the future
        if path[0] == "/":
            self.prefix = path
        else:
            self.prefix += "/" + path
        self.prefix = self.prefix.strip("/")
        return self.prefix

    def pushd(self, path=None):
        self.stack.append(self.prefix)
        return self.cd(path)

    def popd(self):
        self.prefix = self.stack.pop()
        return self.prefix

    def fullpath(self, suffix=None):
        suffix = str(suffix or "")
        if len(suffix) > 0 and suffix[0] == "/":
            retval = suffix
        else:
            retval = ""
            if len(self.prefix) > 0:
                retval += "/" + self.prefix
            if len(suffix) > 0:
                retval += "/" + suffix
        return retval


class ApiObject(object):
    def __init__(self, url, auth, tracker=None, session=None):
        self.baseurl = url.rstrip("/")
        self.auth = auth
        if tracker:
            self.tracker = tracker
        else:
            self.tracker = PathTracker()
        if session:
            self.session = session
        else:
            self.session = Helpers.session_add_retry_handler()

    def __str__(self):
        return '%s "%s" "%s"' % (str(type(self)), self.baseurl, self.tracker.fullpath())

    def clone(self):
        new1 = ApiObject(self.baseurl, self.auth, self.tracker.clone(), self.session)
        return new1

    def cd(self, path=None):
        self.tracker.cd(path)
        return self.compute_url()

    def pushd(self, path=None):
        self.tracker.pushd(path)
        return self.compute_url()

    def popd(self):
        self.tracker.popd()
        return self.compute_url()

    def chroot(self, suffix=""):
        suffix = self.tracker.fullpath(suffix)
        if suffix:
            self.baseurl += suffix
            self.tracker.reset()
        return self.compute_url()

    def collate_pages(self, get_request, data):
        """Given a GET request whose results span across multiple pages, crawl
        each page and collate the results.

        :param first_page_data: "results" dict for first pageful of data
        :type first_page_data: dict
        :param request: Request used to get first page
        :type request: requests.PreparedRequest
        """
        # First, sanity check that all is good. Only process GET requests:
        if get_request.method.upper().strip() != "GET":
            return data

        # Only attempt to pull subsequent pages if we can verify that there are
        # subsequent pages "to be pulled"...
        if isinstance(data, dict) and "results" in data.keys():
            collated_data = data["results"]
            while "nextPage" in data.keys() and data["nextPage"]:
                # Get the next page full of data.
                next_page = self.session.get(
                    get_request.url,
                    params={"pageNo": int(data["pageNo"]) + 1},
                    headers=get_request.headers,
                )

                if not next_page.ok:
                    # Return an empty result.
                    collated_data = []
                    break

                data = next_page.json()
                collated_data = collated_data + data["results"]
                del next_page

            # Dismantle the URL to see if data was requested in descending
            # order...
            query_params = dict(
                urlparse.parse_qsl(urlparse.urlsplit(get_request.url).query)
            )
            descending_order = (
                "isDescendingOrder" in query_params.keys()
                and query_params["isDescendingOrder"]
            )

            # Re-create the final data set as if it were a single page
            # containing all records to ensure that existing stuff that expects
            # this data structure doesn't fall over.
            return {
                "results": collated_data,
                "totalResults": len(collated_data),
                "pageNo": 1,
                "pageSize": len(collated_data),
                "nextPage": False,
                "previousPageNo": 0,
                "descendingOrder": descending_order,
            }
        else:
            return data

    def compute_url(self, suffix=""):
        retval = self.baseurl
        suffix = self.tracker.fullpath(suffix)
        if suffix:
            retval += suffix
        retval = retval.rstrip("/")
        LOG.debug(retval)
        return retval

    def prep_headers(self, headers):
        if not headers:
            return self.auth
        hdr = {}
        hdr.update(self.auth)
        hdr.update(headers)
        return hdr

    def process_result(self, url, resp):
        hstatus = int(resp.status_code)
        if hstatus < 200 or hstatus >= 300:
            msg = "%s %s %s %s" % (resp, resp.request.method, url, resp.content)
            LOG.debug(msg)
            raise RuntimeError(msg)
        try:
            data = resp.json()
            # Some GET requests return paginated output. If all the data fits
            # in one page, return just the contents of the "results" list,
            # otherwise, we need to do an assembly job to collate the entire
            # list of results from all pages and return the full list.
            if (
                resp.request.method == "GET"
                and isinstance(data, dict)
                and data.get("nextPage", None)
            ):
                return self.collate_pages(resp.request, data=data)
            else:
                return data
        except JSONDecodeError:
            return resp.text

    def get(self, suffix=None, headers=None):
        url = self.compute_url(suffix)
        hdr = self.prep_headers(headers)
        resp = self.session.get(url, headers=hdr)
        return self.process_result(url, resp)

    def post(self, suffix=None, headers=None, data=None, json=None, files=None):
        url = self.compute_url(suffix)
        hdr = self.prep_headers(headers)
        resp = self.session.post(url, headers=hdr, data=data, json=json, files=files)
        return self.process_result(url, resp)

    def put(self, suffix=None, headers=None, data=None, json=None):
        url = self.compute_url(suffix)
        hdr = self.prep_headers(headers)
        resp = self.session.put(url, headers=hdr, data=data, json=json)
        return self.process_result(url, resp)

    def delete(self, suffix=None, headers=None, data=None, json=None):
        url = self.compute_url(suffix)
        hdr = self.prep_headers(headers)
        resp = self.session.delete(url, headers=hdr, data=data, json=json)
        return self.process_result(url, resp)

    def patch(self, suffix=None, headers=None, data=None, json=None):
        url = self.compute_url(suffix)
        hdr = self.prep_headers(headers)
        resp = self.session.patch(url, headers=hdr, data=data, json=json)
        return self.process_result(url, resp)


class ApiWrapper(object):
    def __init__(self, apiobject, suffix=None):
        self.api = apiobject.clone()
        if suffix:
            self.api.chroot(suffix)

    def __str__(self):
        return "%s %s" % (str(type(self)), self.api)

    @property
    def session(self):
        return self.api.session

    @session.setter
    def session(self, value):
        LOG.debug(value)
        self.api.session = value

    def get(self, suffix=None, headers=None):
        return self.api.get(suffix, headers=headers)

    def post(self, suffix=None, headers=None, data=None, json=None, files=None):
        return self.api.post(suffix, headers=headers, data=data, json=json, files=files)

    def put(self, suffix=None, headers=None, data=None, json=None):
        return self.api.put(suffix, headers=headers, data=data, json=json)

    def delete(self, suffix=None, headers=None, data=None, json=None):
        return self.api.delete(suffix, headers=headers, data=data, json=json)

    def patch(self, suffix=None, headers=None, data=None, json=None):
        return self.api.patch(suffix, headers=headers, data=data, json=json)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# api.py
# OpsRamp-specific variant of ApiWrapper base class as a container for
# some methods and helpers that are common to multiple parts of that API.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64

from opsramp.base import ApiWrapper


class ORapi(ApiWrapper):
    """OpsRamp-specific variant of ApiWrapper base class as a container for
    some methods and helpers that are common to multiple parts of that API."""

    # Returns a string containing a base64 encoded version of the
    # content of the specified file. It was quite finicky to come
    # up with a method that works on both Python 2 and 3 so please
    # don't modify this, or test carefully if you do.
    @staticmethod
    def b64encode_payload(fname):
        with open(fname, "rb") as f:
            content = base64.b64encode(f.read())
        return content.decode()

    def search(self, pattern="", headers=None, suffix="search"):
        if pattern:
            if pattern[0] != "?":
                pattern = "?" + pattern
            suffix += pattern
        return super(ORapi, self).get(suffix, headers)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# globalconfig.py
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class GlobalConfig(ORapi):
    def __init__(self, parent):
        super(GlobalConfig, self).__init__(parent.api, "")

    def get_alert_types(self):
        return self.api.get("/alertTypes")

    def get_countries(self):
        return self.api.get("/cfg/countries")

    def get_timezones(self):
        return self.api.get("/cfg/timezones")

    def get_alert_technologies(self):
        return self.api.get("/cfg/alertTechnologies")

    def get_channels(self):
        return self.api.get("/cfg/tenants/channels")

    def get_nocs(self):
        # Bizarrely this API call throws a 500 error if there are
        # no NOCs defined. Handle it gracefully.
        try:
            retval = self.api.get("/cfg/tenants/nocs")
        except RuntimeError as e:
            if '"code":"0005"' in str(e):
                retval = []
            else:
                raise
        return retval

    def get_device_types(self):
        return self.api.get("/cfg/devices/types")



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding

tnt_id = os.environ["OPSRAMP_TENANT_ID"]


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    ormp = connect()
    tnt = ormp.tenant(tnt_id)
    service_maps = tnt.service_maps()

    top_level_json = [
        {
            "name": "Finance Inventory",
            "createdDate": "2018-06-20T11:33:08+0000",
            "updatedDate": "2018-06-20T11:33:08+0000",
            "childType": "SERVICE",
            "frequency": 5,
            "thresholdType": "count",
            "thresholdLimit": 1,
            "monitorNames": ["service.availability.metric"],
            "include": "ALL",
            "type": "alert",
            "alertType": 2,
            "alert": False,
            "metrics": [],
        }
    ]

    resp = service_maps.create(top_level_json)

    top_level_sgid = resp[0]["id"]

    # Now create two sub-maps attached to the top level one above.

    second_level_json_dept_a = [
        {
            "name": "Department A",
            "createdDate": "2018-06-20T11:33:08+0000",
            "updatedDate": "2018-06-20T11:33:08+0000",
            "childType": "SERVICE",
            "frequency": 5,
            "thresholdType": "count",
            "thresholdLimit": 1,
            "monitorNames": ["service.availability.metric"],
            "include": "ALL",
            "type": "alert",
            "alertType": 2,
            "alert": False,
            "metrics": [],
            "parent": {"id": top_level_sgid},
        }
    ]

    second_level_json_dept_b = [
        {
            "name": "Department B",
            "createdDate": "2018-06-20T11:33:08+0000",
            "updatedDate": "2018-06-20T11:33:08+0000",
            "childType": "SERVICE",
            "frequency": 5,
            "thresholdType": "count",
            "thresholdLimit": 1,
            "monitorNames": ["service.availability.metric"],
            "include": "ALL",
            "type": "alert",
            "alertType": 2,
            "alert": False,
            "metrics": [],
            "parent": {"id": top_level_sgid},
        }
    ]

    resp = service_maps.create(second_level_json_dept_a)
    second_level_sgid_dept_a = resp[0]["id"]

    resp = service_maps.create(second_level_json_dept_b)
    second_level_sgid_dept_b = resp[0]["id"]

    print("dept a %s dept b %s" % (second_level_sgid_dept_a, second_level_sgid_dept_b))

    vm_map_json = [
        {
            "name": "Warehouse Inventory",
            "createdDate": "2018-06-20T11:33:08+0000",
            "updatedDate": "2018-06-20T11:33:08+0000",
            "childType": "DEVICE",
            "frequency": 5,
            "thresholdType": "count",
            "thresholdLimit": 1,
            "monitorNames": ["service.availability.metric"],
            "include": "ALL",
            "type": "alert",
            "alertType": 2,
            "alert": False,
            "metrics": [],
            "filterCriteria": {
                "matchType": "ANY",
                "rules": [
                    {
                        "key": "Device Name",
                        "operator": "Contains",
                        "value": "Linux",
                        "resourceType": "DEVICE",
                    }
                ],
            },
            "parent": {"id": second_level_sgid_dept_b},
        }
    ]

    resp = service_maps.create(vm_map_json)
    vm_map_sgid = resp[0]["id"]

    print("vm map sgid %s" % vm_map_sgid)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding
import yaml


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ["OPSRAMP_TENANT_ID"]

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    group = tenant.integrations().instances()

    # Create new Email Alerts Integration...
    jdata = {
        "displayName": "Test Email Alerts integration",
        "emailProps": [
            {
                "name": "logz.io email alert",
                "identifier": "^\\[Alert\\]",
                "identifierSource": "EMAIL_SUBJECT",
                "properties": [
                    {
                        "name": "Alert State",
                        "defaultValue": "OK",
                        "condition": {
                            "contentSource": "EMAIL_SUBJECT",
                            "operator": "BETWEEN",
                            "startValue": "\\(",
                            "endValue": " severity\\)",
                        },
                        "propertyMappings": {
                            "attrValues": [
                                {
                                    "attrValue": "Critical",
                                    "thirdPartyAttrValue": "Severe",
                                },
                                {"attrValue": "Warning", "thirdPartyAttrValue": "High"},
                                {
                                    "attrValue": "Observed",
                                    "thirdPartyAttrValue": "Medium",
                                },
                                {"attrValue": "Info", "thirdPartyAttrValue": "Low"},
                                {"attrValue": "Ok", "thirdPartyAttrValue": "Info"},
                            ]
                        },
                    },
                    {
                        "name": "Service Name",
                        "defaultValue": "whatever",
                        "condition": {"contentSource": "DEFAULT_VALUE"},
                    },
                    {
                        "name": "Device Host Name",
                        "defaultValue": "whatever",
                        "condition": {"contentSource": "DEFAULT_VALUE"},
                    },
                ],
            }
        ],
    }

    print("create integration using this payload")
    print(yaml.dump(jdata, default_flow_style=False))

    resp = group.create("Email Alerts", jdata)
    print("result:")
    print(yaml.dump(resp, default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class DevmgmtTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

    def test_policies(self):
        group = self.client.policies()
        assert group
        with requests_mock.Mocker() as m:
            # The search function in this class supports two search
            # syntaxes so test both.
            display_name = "whatever"
            expected = {"id": "abcdefg", "name": display_name}
            url = group.api.compute_url("search?name=%s" % display_name)
            m.get(url, json=expected, complete_qs=True)
            # The old simple search syntax.
            actual = group.search(display_name)
            assert actual == expected
            # The newer query syntax.
            actual = group.search("name=%s" % display_name)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 111111
            expected = {"id": thisid}
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 123456
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.put(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 789012
            expected = {"id": thisid}
            url = group.api.compute_url("%s/action/run" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.run(uuid=thisid)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 345678
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected

    def test_discovery(self):
        group = self.client.discovery()
        assert group
        with requests_mock.Mocker() as m:
            # The search function in this class supports two search
            # syntaxes so test both.
            display_name = "whatever"
            expected = {"id": "abcdefg", "name": display_name}
            url = group.api.compute_url("search?name=%s" % display_name)
            m.get(url, json=expected, complete_qs=True)
            # The old simple search syntax.
            actual = group.search(display_name)
            assert actual == expected
            # The newer query syntax.
            actual = group.search("name=%s" % display_name)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 111111
            expected = {"id": thisid}
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 123456
            expected = {"id": thisid}
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 789012
            expected = {"id": thisid}
            url = group.api.compute_url("action/scan/%s" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.rescan(uuid=thisid)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 345678
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected

    def test_credential_sets(self):
        group = self.client.credential_sets()
        assert group
        with requests_mock.Mocker() as m:
            thisid = 111111
            expected = {"id": thisid}
            url = group.api.compute_url()
            m.get(url, json=expected, complete_qs=True)
            actual = group.get()
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 222222
            expected = {"id": thisid}
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 123456
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(uuid=thisid)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 789012
            expected = {"id": thisid}
            url = group.api.compute_url("%s/minimal" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(uuid=thisid, minimal=True)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 345678
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=expected)
            assert actual == expected

        with requests_mock.Mocker() as m:
            thisid = 901234
            expected = {"id": thisid}
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.group = self.client.mgmt_profiles()
        assert "Profiles" in str(self.group)

    def test_search(self):
        thisid = 111111
        expected = {"id": thisid}
        pattern = "whatever"
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url("search?%s" % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = self.group.search(pattern)
            assert actual == expected

    def test_create(self):
        thisid = 345678
        expected = {"id": thisid}
        fake_defn = {"name": "jack"}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = self.group.create(definition=fake_defn)
            assert actual == expected

    def test_update(self):
        thisid = 123456
        expected = {"id": thisid}
        fake_defn = {"name": "mrs doyle"}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = self.group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_delete(self):
        thisid = 789012
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = self.group.delete(uuid=thisid)
            assert actual == expected

    def test_attach(self):
        thisid = 345678
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url("%s/attach" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = self.group.attach(uuid=thisid)
            assert actual == expected

    def test_detach(self):
        thisid = 901234
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url("%s/detach" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = self.group.detach(uuid=thisid)
            assert actual == expected

    def test_reconnect(self):
        thisid = 901234
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = self.group.api.compute_url("%s/reconnectTunnel" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = self.group.reconnect(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import copy
import unittest

import opsramp.binding
from opsramp.integrations import Instances
import requests_mock


class InstancesTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.integs = self.client.integrations()
        assert "Integrations" in str(self.integs)

    def test_itypes(self):
        group = self.integs.itypes()
        thisid = 111111
        expected = {"id": thisid}
        pattern = "whatever"
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("search?%s" % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
            assert actual == expected
        # check the compatibility function would return the
        # same type of object, so there's no need to repeat
        # all of the tests for it.
        assert isinstance(self.integs.available(), type(group))

    def test_instances(self):
        group = self.integs.instances()
        thisid = 222222
        expected = {"id": thisid}
        pattern = "whatever"
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("search?%s" % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
            assert actual == expected
        # check the compatibility function would return the
        # same type of object, so there's no need to repeat
        # all of the tests for it.
        assert isinstance(self.integs.installed(), type(group))

    def test_instance_kubernetes_configuration(self):
        group = self.integs.instances()
        thisid = 123456
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/configFile/Kubernetes" % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.get_kubernetes_configuration(uuid=thisid)
            assert actual == expected

    def test_instance_create(self):
        group = self.integs.instances()
        thisid = 789012
        expected = {"id": thisid}
        name = "unit-test-integration"
        fake_defn = {"name": "fr larry duff"}
        with requests_mock.Mocker() as m:
            url = group.creator_api.compute_url(name)
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(type_name=name, definition=fake_defn)
            assert actual == expected

    def test_instance_update(self):
        group = self.integs.instances()
        thisid = 123456
        expected = {"id": thisid}
        fake_defn = {"name": "fr paul stone"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_instance_enable(self):
        group = self.integs.instances()
        thisid = 789012
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/enable" % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.enable(uuid=thisid)
            assert actual == expected

    def test_instance_disable(self):
        group = self.integs.instances()
        thisid = 345678
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/disable" % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.disable(uuid=thisid)
            assert actual == expected

    def test_instance_delete(self):
        group = self.integs.instances()
        thisid = 999999
        url = group.api.compute_url("%s" % thisid)

        expected_response = {"id": thisid}

        # Test that the delete function inserts a default reason
        # if we don't provide one.
        expected_send = {"uninstallReason": "<Not specified>"}
        with requests_mock.Mocker() as m:
            adapter = m.delete(url, json=expected_response, complete_qs=True)
            actual_response = group.delete(uuid=thisid)
            assert adapter.last_request.json() == expected_send
            assert actual_response == expected_response

        # Test that the delete function sends the reason we specified.
        delete_reason = "Totally fake reason to delete something"
        expected_send = {"uninstallReason": delete_reason}
        with requests_mock.Mocker() as m:
            adapter = m.delete(url, json=expected_response, complete_qs=True)
            actual_response = group.delete(uuid=thisid, uninstall_reason=delete_reason)
            assert adapter.last_request.json() == expected_send
            assert actual_response == expected_response

    def test_instance_notifier(self):
        group = self.integs.instances()
        thisid = 901234
        expected = {"id": thisid}
        fake_defn = {"name": "fr fintan stack"}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url("%s/notifier" % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.notifier(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_instance_auth_type(self):
        group = self.integs.instances()
        thisid = 567890
        url = group.api.compute_url("%s/inbound/authentication" % thisid)
        for key in "OAUTH2", "WEBHOOK", "BASIC":
            expected = {"type": key}
            with requests_mock.Mocker() as m:
                m.post(url, json=expected, complete_qs=True)
                actual = group.set_auth_type(uuid=thisid, auth_type=key)
                assert actual == expected
        with requests_mock.Mocker() as m:
            with self.assertRaises(AssertionError):
                group.set_auth_type(uuid=thisid, auth_type="unit test value")
            assert m.call_count == 0

    def base_display_name(self, fn):
        with self.assertRaises(AssertionError):
            fn(display_name=None)
        testname = "Unit test 1"
        actual = fn(display_name=testname)
        expected = {"displayName": testname}
        assert actual == expected

    def base_logo(self, fn):
        testname = "Unit test 2"
        testfile = "README.md"
        with self.assertRaises(IOError):
            fn(display_name=testname, logo_fname=testfile + ".that.doesnt.exist")
        actual = fn(display_name=testname, logo_fname=testfile)
        encoded = actual["logo"]["file"]
        cleartext = base64.b64decode(encoded)
        with open(testfile, "rb") as f:
            content = f.read()
        assert content == cleartext

        expected = {
            "displayName": testname,
            "logo": {"name": testfile, "file": encoded},
        }
        assert actual == expected

    def test_base(self):
        self.base_display_name(Instances.mkBase)
        self.base_logo(Instances.mkBase)

    def test_emailalert(self):
        self.base_display_name(Instances.mkEmailAlert)
        self.base_logo(Instances.mkEmailAlert)

    def test_custom(self):
        self.base_display_name(Instances.mkCustom)
        self.base_logo(Instances.mkCustom)
        with self.assertRaises(AssertionError):
            Instances.mkCustom(
                display_name="bad auth type", inbound_auth_type="BAD UNIT TEST VALUE"
            )
        # Now a variant that should work.
        testname = "Custom integration unit test"
        fakeuuid = "deadbeef"
        auth_type = "OAUTH2"
        actual = Instances.mkCustom(
            display_name=testname, parent_uuid=fakeuuid, inbound_auth_type=auth_type
        )
        expected = {
            "displayName": testname,
            "parentIntg": {"id": fakeuuid},
            "inboundConfig": {"authentication": {"authType": auth_type}},
        }
        assert actual == expected
        # exercise the auth_type helper while we have a suitable struct.
        atypes = Instances.auth_type(actual)
        assert atypes == (auth_type, None)

    def test_ARM(self):
        testname = "Azure ARM integration unit test"
        subscription_id = "123456789"
        tenant_id = "Some random Azure tenant"
        client_id = "Azure client shtuff"
        secret_key = "Open Sesame I Say!"
        actual = Instances.mkAzureARM(
            display_name=testname,
            arm_subscription_id=subscription_id,
            arm_tenant_id=tenant_id,
            arm_client_id=client_id,
            arm_secret_key=secret_key,
        )
        expected = {
            "displayName": testname,
            "credential": {
                "credentialType": "AZURE",
                "AzureType": "ARM",
                "SubscriptionId": subscription_id,
                "TenantId": tenant_id,
                "ClientID": client_id,
                "SecretKey": secret_key,
            },
        }
        assert actual == expected

    def test_ASM(self):
        testname = "Azure ASM integration unit test"
        subscription_id = "ABCDEFGHIJKLMNOP"
        mgmt_cert = "Azure management cert"
        keystore_pass = "Abrakedabra"
        actual = Instances.mkAzureASM(
            display_name=testname,
            arm_subscription_id=subscription_id,
            arm_mgmt_cert=mgmt_cert,
            arm_keystore_pass=keystore_pass,
        )
        expected = {
            "displayName": testname,
            "credential": {
                "credentialType": "AZURE",
                "AzureType": "ASM",
                "SubscriptionId": subscription_id,
                "ManagementCertificate": mgmt_cert,
                "KeystorePassword": keystore_pass,
            },
        }
        assert actual == expected

    def test_valuemap(self):
        expected = [
            {"attrValue": "client_113", "tenantAttrValue": "22cdbc5bb401d737b088c9"},
            {"attrValue": "client_843", "tenantAttrValue": "66cdbc5bb401d737b088c9"},
        ]
        actual = Instances.mkValueMap(
            ("attrValue", "tenantAttrValue"),
            (
                ("client_113", "22cdbc5bb401d737b088c9"),
                ("client_843", "66cdbc5bb401d737b088c9"),
            ),
        )
        assert actual == expected

        expected = [
            {"attrValue": "Fido", "thirdPartyAttrValue": "His Royal Fidoness"},
            {"attrValue": "brown", "thirdPartyAttrValue": "Autumn Sunset"},
        ]
        actual = Instances.mkValueMap(
            ("attrValue", "thirdPartyAttrValue"),
            (("Fido", "His Royal Fidoness"), ("brown", "Autumn Sunset")),
        )
        assert actual == expected

    def test_basenotifier(self):
        expected = {
            "type": "REST_API",
            "baseURI": "www.example.com",
            "authType": "OAUTH2",
            "grantType": "CLIENT_CREDENTIALS",
            "accessTokenURI": "www.example.com/creds",
            "apiKey": "6h67PAAFscVPMwhQZFcshpcqN5b6pyU9",
            "apiSecret": "totalandcompletenonsense",
        }
        actual = Instances.mkBaseNotifier(
            expected["type"],
            expected["baseURI"],
            access_token_uri=expected["accessTokenURI"],
            api_key=expected["apiKey"],
            api_secret=expected["apiSecret"],
        )
        assert actual == expected

        expected = {
            "type": "SOAP_API",
            "baseURI": "www.example.com",
            "authType": "NONE",
        }
        actual = Instances.mkBaseNotifier(
            expected["type"], expected["baseURI"], auth_type=expected["authType"]
        )
        assert actual == expected

        expected = {
            "type": "REST_API",
            "baseURI": "www.example.com/whatevs",
            "authType": "BASIC",
            "grantType": "PASSWORD",
            "userName": "elvis",
            "password": "graceland",
        }
        actual = Instances.mkBaseNotifier(
            expected["type"],
            expected["baseURI"],
            auth_type=expected["authType"],
            grant_type=expected["grantType"],
            user_name=expected["userName"],
            password=expected["password"],
        )
        assert actual == expected

    def test_redaction(self):
        original = {
            "inboundConfig": {"authentication": {"token": "shakespeare"}},
            "outboundConfig": {
                "authentication": {
                    "apiKeyPairs": [{"char1": "juliet", "char2": "goneril"}]
                }
            },
        }
        # make a copy and check that it really is a copy.
        tvalues = copy.deepcopy(original)
        assert tvalues is not original
        assert tvalues == original
        # the redact function should modify the dict in place, not copy it.
        result = Instances.redact_response(tvalues)
        assert result is tvalues
        # check the redaction did what we expected.
        original["inboundConfig"]["authentication"]["token"] = "REDACTED"
        original["outboundConfig"]["authentication"]["apiKeyPairs"][0][
            "char1"
        ] = "REDACTED"
        original["outboundConfig"]["authentication"]["apiKeyPairs"][0][
            "char2"
        ] = "REDACTED"
        assert result is not original
        assert result == original



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# binding.py
# Defines the primary entry points for callers of this library.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi
from opsramp.base import ApiObject
from opsramp.globalconfig import GlobalConfig
from opsramp.metrics import MetricsApi
from opsramp.tenant import Tenant


def connect(url, key, secret):
    auth_url = url + "/auth/oauth/token"
    auth_hdrs = {
        "Content-Type": "application/x-www-form-urlencoded",
        "Accept": "application/json,application/xml",
    }
    body = (
        "grant_type=client_credentials&"
        "client_id=%s&"
        "client_secret=%s" % (key, secret)
    )
    ao = ApiObject(auth_url, auth_hdrs)
    auth_resp = ao.post(data=body)
    token = auth_resp["access_token"]
    return Opsramp(url, token)


class Opsramp(ORapi):
    def __init__(self, url, token):
        self.auth = {
            "Authorization": "Bearer " + token,
            "Accept": "application/json,application/xml",
        }
        apiobject = ApiObject(url + "/api/v2", self.auth)
        super(Opsramp, self).__init__(apiobject)

    def __str__(self):
        return "%s %s" % (str(type(self)), self.api)

    def config(self):
        return GlobalConfig(self)

    def tenant(self, name):
        return Tenant(self, name)

    def metrics(self):
        return MetricsApi(self)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# resources.py
# Resource classes.
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


def list2ormp(result_obj):
    """Bizarrely, OpsRamp sometimes returns a simple list for
    Resources API calls instead of its usual results struct."""
    if isinstance(result_obj, dict):
        assert "results" in result_obj
        assert "totalResults" in result_obj
        return result_obj

    # If there are no results, the response may be completely
    # empty, which maps to `result_obj` being an empty string here.
    if result_obj == "":
        result_obj = []

    # Wrap it up in a fake of the typical OpsRamp results struct
    # so that callers don't have to special case it.
    assert isinstance(result_obj, list)
    count = len(result_obj)
    retval = {
        "totalResults": count,
        "pageSize": count,
        "totalPages": 1,
        "pageNo": 1,
        "previousPageNo": 0,
        "nextPage": False,
        "descendingOrder": False,
        "results": result_obj,
    }
    return retval


class Resources(ORapi):
    def __init__(self, parent):
        super(Resources, self).__init__(parent.api, "resources")

    def create(self, definition):
        url_suffix = ""
        return self.api.post(url_suffix, json=definition)

    def update(self, uuid, definition):
        url_suffix = uuid
        return self.api.post(url_suffix, json=definition)

    def delete(self, uuid):
        url_suffix = uuid
        return self.api.delete(url_suffix)

    def search(self, pattern=""):
        """returns *verbose* details about resources on this tenant"""
        simple_list = super(Resources, self).search(pattern=pattern, suffix="search")
        return list2ormp(simple_list)

    def minimal(self, pattern=""):
        """returns *minimal* details about resources on this tenant"""
        simple_list = super(Resources, self).search(pattern=pattern, suffix="minimal")
        return list2ormp(simple_list)

    def applications(self, uuid):
        url_suffix = "{0}/applications".format(uuid)
        simple_list = self.api.get(url_suffix)
        return list2ormp(simple_list)

    def availability(self, uuid, start_epoch, end_epoch):
        url_suffix = "{0}/availability?startTime={1}&endTime={2}".format(
            uuid, start_epoch, end_epoch
        )
        return self.api.get(url_suffix)

    def get_templates(self, uuid, pattern=None):
        url_suffix = "{0}/templates".format(uuid)
        self.api.pushd(url_suffix)
        resp = super(Resources, self).search(pattern)
        resp = self.search(pattern)
        self.api.popd()
        return resp



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import mock
from mock import MagicMock
import opsramp.base
import requests
from requests import codes as http_status
import requests_mock
import simplejson


class FakeResp(object):
    def __init__(self, content, request, status_code):
        self.status_code = status_code
        self.content = content
        # We are deliberately using simplejson (instead of the default json) because
        # that's what the requests module uses and we want to raise the same
        # types of exceptions so that our tests exercise the right code paths.
        self.text = simplejson.dumps(self.content)
        self.json_fail = False
        self.request = request

    def json(self):
        content = self.content
        if self.json_fail:
            # deliberately cause a parsing exception
            content = "deliberately bad JSON for unit test purposes!!"
        return content


class ApiObjectTest(unittest.TestCase):
    def setUp(self):
        self.fake_url = "mock://api.example.com"
        self.fake_token = "ffffffffffffffff"
        self.fake_auth = {
            "Authorization": "Bearer %s" % self.fake_token,
            "Accept": "application/json",
        }
        self.ao = opsramp.base.ApiObject(self.fake_url, self.fake_auth.copy())
        assert "ApiObject" in str(self.ao)

    def test_default_verbs(self):
        dv = opsramp.base.Helpers.default_retry_verbs()
        assert isinstance(dv, frozenset)
        assert dv == requests.packages.urllib3.util.Retry.DEFAULT_ALLOWED_METHODS

    @mock.patch("opsramp.base.Helpers.retryclass")
    def test_default_verbs_fallback(self, mock_retryclass):
        # mock that the new name is missing, but the old one exists.
        del mock_retryclass.DEFAULT_ALLOWED_METHODS
        fake_whitelist = ["unit", "test", "stuff"]
        mock_retryclass.DEFAULT_METHOD_WHITELIST = fake_whitelist
        # check that the Helpers class uses the old name.
        dv = opsramp.base.Helpers.default_retry_verbs()
        assert dv == fake_whitelist

    def test_shared_session(self):
        """All clones should share the requests session"""
        ao2 = self.ao.clone()
        assert ao2.session is self.ao.session

    def test_compute_url(self):
        # some value, doesn't really matter.
        suffix = "unit/test/value"
        expected = self.ao.baseurl + "/" + suffix
        assert self.ao.compute_url(suffix) == expected
        # empty string and None should both return the root path.
        rootdir = self.ao.baseurl
        assert self.ao.compute_url() == rootdir
        assert self.ao.compute_url("/") == rootdir
        assert self.ao.compute_url("") == rootdir
        assert self.ao.compute_url(None) == rootdir

    # We're not testing an exhaustive set of suffix patterns here because
    # that is already being done by the PathTracker unit tests.
    def test_cd(self):
        # some value, doesn't really matter.
        suffix = "unit/test/cd"
        expected = self.ao.compute_url() + "/" + suffix
        actual = self.ao.cd(suffix)
        assert actual == expected
        assert self.ao.compute_url() == expected
        # check the root path works.
        rootdir = self.ao.baseurl
        self.ao.cd("/random/place1")
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd("/") == rootdir
        assert self.ao.compute_url() == rootdir
        # no args should default to the root path.
        self.ao.cd("/random/place0")
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd() == rootdir
        assert self.ao.compute_url() == rootdir
        # empty string should default to the root path.
        self.ao.cd("/random/place2")
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd("") == rootdir
        assert self.ao.compute_url() == rootdir
        # None should default to the root path.
        self.ao.cd("/random/place3")
        assert self.ao.cd(None) == rootdir
        assert self.ao.compute_url() == rootdir

    # We're not testing an exhaustive set of suffix patterns here because
    # that is already being done by the PathTracker unit tests.
    def test_pushpopd(self):
        suffix = "unit/test/pushd"
        base = self.ao.compute_url()
        expected = base + "/" + suffix
        actual = self.ao.pushd(suffix)
        assert actual == expected
        assert self.ao.compute_url() == expected
        actual = self.ao.popd()
        assert actual == base
        assert self.ao.compute_url() == base

    def test_headers(self):
        expected = self.fake_auth.copy()
        actual = self.ao.prep_headers({})
        assert actual == expected

        expected = self.fake_auth.copy()
        custom = {"color": "pink", "distance": "far"}
        expected.update(custom)
        actual = self.ao.prep_headers(custom)
        assert actual == expected

        # The caller is supposed to be able to override any of the standard
        # auth headers by providing its own value. Check this works.
        expected = self.fake_auth.copy()
        key = "Authorization"
        assert key in expected
        testvalue = "open sesame"
        assert expected[key] != testvalue
        custom[key] = testvalue
        actual = self.ao.prep_headers(custom)
        assert actual[key] == testvalue
        expected.update(custom)
        assert actual == expected

    def test_result_ok(self):
        with requests_mock.mock() as m:
            expected = {"hello": "world"}
            m.get(self.fake_url, json=expected, status_code=http_status.OK)

            faked_response = FakeResp(
                content=expected,
                request=self.ao.session.get(self.fake_url).request,
                status_code=http_status.OK,
            )

            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is dict
            assert actual == expected

    def test_result_created(self):
        with requests_mock.mock() as m:
            expected = {"hello": "world"}
            m.get(self.fake_url, json=expected, status_code=http_status.OK)

            faked_response = FakeResp(
                content=expected,
                request=self.ao.session.get(self.fake_url).request,
                status_code=http_status.CREATED,
            )

            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is dict
            assert actual == expected

    def test_result_invalid_json(self):
        with requests_mock.mock() as m:
            expected = "deliberately bad JSON for unit test purposes!!"
            m.get(self.fake_url, text=expected)

            faked_response = FakeResp(
                content=["nothing"],
                request=self.ao.session.get(self.fake_url).request,
                status_code=http_status.OK,
            )
            faked_response.json_fail = True

            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is str
            assert actual == expected

    def test_result_failure(self):
        with requests_mock.mock() as m:
            m.get(
                self.fake_url,
                status_code=http_status.BAD_REQUEST,
                text="Pretending to fail horribly",
            )

            faked_response = FakeResp(
                content=["nothing"],
                request=self.ao.session.get(self.fake_url).request,
                status_code=http_status.BAD_REQUEST,
            )

            with self.assertRaises(RuntimeError):
                self.ao.process_result(self.fake_url, faked_response)

    def test_paginated_results(self):
        # Define three pages of test data that return the following result
        # data:
        # Page 1: 1-10
        # Page 2: 11-20
        # Page 3: 21-30
        # If the pagination works, the result should be the range 1-30.
        request_responses = []
        for result_page in range(3):
            result_data = list(range((10 * result_page) + 1, (10 * result_page) + 11))

            response_body = {
                "results": result_data,
                "totalResults": 30,
                "pageNo": result_page + 1,
                "pageSize": 10,
                "nextPage": (result_page < 2),
                "previousPageNo": result_page,
                "descendingOrder": False,
            }

            response = {
                "status_code": http_status.OK,
                "json": response_body,
            }

            request_responses.append(response)

        # Build up the list of expected result data based on what we're feeding
        # in to the mock.
        expected = []
        for response in request_responses:
            expected = expected + response["json"]["results"]

        with requests_mock.mock() as m:
            # Feed the mock with the three responses it should return
            m.get(self.fake_url, request_responses)

            # Build the initial (faked) response based on the above
            faked_response = FakeResp(
                content=request_responses[0]["json"],
                request=self.ao.session.get(self.fake_url).request,
                status_code=http_status.OK,
            )
            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is dict

            # So that no existing code is broken, the behaviour emulates the
            # existing behaviour of the API except that it acts as if the data
            # was in one big page.
            assert actual["results"] == expected
            assert actual["totalResults"] == len(expected)
            assert actual["nextPage"] is False
            assert actual["pageNo"] == 1
            assert actual["previousPageNo"] == 0

        # Test that if the request is not a HTTP GET, complain:
        fake_get_request = MagicMock()
        fake_get_request.method = "FAKE"
        expected = ["request is not GET"]
        actual = self.ao.collate_pages(fake_get_request, expected)
        assert actual == expected

        # Test that it handles the lack of a "results" fields in the data
        # correctly
        fake_get_request.method = "GET"
        expected = ["request does not contain results key"]
        actual = self.ao.collate_pages(fake_get_request, expected)
        assert actual == expected

        # Test that if the next page is not retrieved successfully, an empty
        # result is returned.
        with requests_mock.mock() as m:
            m.get(
                self.fake_url,
                status_code=http_status.BAD_REQUEST,
                json=["should not be seen"],
            )

            fake_get_request = MagicMock()
            fake_get_request.method = "GET"
            fake_get_request.url = self.fake_url

            page_1_data = {
                "results": ["should not be seen"],
                "nextPage": True,
                "pageNo": 1,
            }

            actual = self.ao.collate_pages(fake_get_request, page_1_data)
            assert actual["results"] == []
            assert actual["totalResults"] == 0
            assert actual["pageSize"] == 0

    def test_get(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = "unit test get result"
            m.get(url, text=expected)
            actual = self.ao.get()
            assert actual == expected

    def test_put(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = "unit test put result"
            m.put(url, text=expected)
            actual = self.ao.put()
            assert actual == expected

    def test_post(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = "unit test post result"
            m.post(url, text=expected)
            actual = self.ao.post()
            assert actual == expected

    def test_delete(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = "unit test delete result"
            m.delete(url, text=expected)
            actual = self.ao.delete()
            assert actual == expected

    def test_patch(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = "unit test patch result"
            m.patch(url, text=expected)
            actual = self.ao.patch()
            assert actual == expected



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# rba.py
# Runbook Automation related classes
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class Rba(ORapi):
    def __init__(self, parent):
        super(Rba, self).__init__(parent.api, "rba")

    def categories(self):
        return Categories(self)


class Categories(ORapi):
    def __init__(self, parent):
        super(Categories, self).__init__(parent.api, "categories")

    # Creates a new category with optional parent.
    def create(self, name, parent_uuid=None):
        jjj = {"name": name}
        if parent_uuid:
            jjj["parent"] = {"id": parent_uuid}
        return self.api.post(json=jjj)

    def category(self, uuid):
        return Category(self, uuid)

    # Updates an RBA category based on its ID.
    def update(self, uuid, definition):
        # Making sure the UUID is specified in the request body as it's not
        # specified in the URL for some reason.
        definition["id"] = uuid
        return self.api.put("", json=definition)

    # Deletes an RBA category based on its ID
    def delete(self, uuid):
        return self.api.delete("%s" % uuid)


class Category(ORapi):
    def __init__(self, parent, uuid):
        super(Category, self).__init__(parent.api, "%s/scripts" % uuid)

    # Creates a script in this category
    def create(self, definition):
        return self.api.post(json=definition)

    # Updates a script given the id
    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    # Deletes a script given by the id
    def delete(self, uuid):
        return self.api.delete("%s" % uuid)

    # A helper function for use with mkParameter & mkScript.
    @staticmethod
    def mkAttachment(name, payload):
        assert name
        assert payload
        return {"name": name, "file": payload}

    # Returns a Python dict that defines one parameter of a script.
    @staticmethod
    def mkParameter(name, description, datatype, optional=False, default=None):
        assert name
        assert description
        assert datatype
        if optional:
            assert default is not None
        return {
            "name": name,
            "description": description,
            "dataType": datatype,
            "type": "OPTIONAL" if optional else "REQUIRED",
            "defaultValue": default,
        }

    # Returns a Python dict that defines a new RBA script. Intended
    # for use with Category.create_script() it also asserts that
    # certain rules are being complied with in the script definition.
    @staticmethod
    def mkScript(
        name,
        description,
        platforms,
        execution_type,
        payload=None,
        payload_file=None,
        parameters=None,
        script_name=None,
        install_timeout=0,
        registry_path=None,
        registry_value=None,
        process_name=None,
        service_name=None,
        output_directory=None,
        output_file=None,
    ):
        assert name
        assert description
        assert platforms
        assert execution_type
        if payload_file:
            assert not payload
            payload = ORapi.b64encode_payload(payload_file)
        assert payload
        if execution_type not in ("DOWNLOAD", "EXE", "MSI"):
            assert not output_directory
            assert not output_file
        if execution_type == "COMMAND":
            payload_key = "command"
            payload_value = payload
        else:
            assert script_name
            payload_key = "attachment"
            payload_value = Category.mkAttachment(script_name, payload)

        # fields that are always present.
        retval = {
            "name": name,
            "description": description,
            "platforms": platforms,
            "executionType": execution_type,
            payload_key: payload_value,
        }

        # optional ones.
        if parameters:
            retval["parameters"] = parameters

        if install_timeout:
            retval["installTimeout"] = install_timeout

        if "WINDOWS" not in platforms:
            assert not registry_path
            assert not registry_value
        else:
            if registry_path:
                retval["registryPath"] = registry_path
                if registry_value:
                    retval["registryValue"] = registry_value

        if "LINUX" not in platforms:
            assert not process_name
            assert not service_name
        else:
            if process_name:
                retval["processName"] = process_name
            if service_name:
                retval["serviceName"] = service_name

        return retval



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import opsramp.resources
import requests_mock


class StaticsTest(unittest.TestCase):
    def test_list2ormp_passthru(self):
        good1 = {"results": "exist", "totalResults": 12}
        result = opsramp.resources.list2ormp(good1)
        assert result is good1

    @staticmethod
    def _canonical_response_struct(lst: list) -> dict:
        assert isinstance(lst, list), lst
        count: int = len(lst)
        return {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": lst,
        }

    def test_list2ormp_conversion(self):
        list_form: list = [1, 2, 3, 4, 5, 6, 7, 8]
        canonical_form: dict = self._canonical_response_struct(list_form)
        actual = opsramp.resources.list2ormp(list_form)
        assert actual == canonical_form

    def test_list2ormp_empty_conversion(self):
        expected: dict = self._canonical_response_struct([])
        actual = opsramp.resources.list2ormp("")
        assert actual == expected

    def test_list2ormp_invalid(self):
        bad_ones = (
            3.14159,
            "string not allowed",
            {},
            {"big cow": "close", "small cow": "far away"},
            {"results": "not enough on their own"},
            {"totalResults": 0, "noresults": "bad"},
        )
        for bad1 in bad_ones:
            with self.assertRaises(AssertionError):
                opsramp.resources.list2ormp(bad1)


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

    def test_create(self):
        group = self.client.resources()
        fake_create_json = {
            "resourceName": "ABBY-PC",
            "hostName": "ABBY-PC",
            "aliasName": "Server PC",
            "resourceType": "server",
            "os": "Ubuntu 14.04.6 LTS",
            "serialNumber": "1234-5678-901234",
        }
        thisid = 333333
        fake_result = {"id": thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.create(definition=fake_create_json)
            assert actual == fake_result

    def test_update(self):
        group = self.client.resources()
        fake_resource_id = "123456"
        fake_update_json = {"os": "Ubuntu 18.04.4 LTS"}
        fake_result = {"id": fake_resource_id}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(fake_resource_id)
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.update(uuid=fake_resource_id, definition=fake_update_json)
            assert actual == fake_result

    def test_delete(self):
        group = self.client.resources()
        fake_resource_id = "789012"
        fake_result = {"id": fake_resource_id}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(fake_resource_id)
            m.delete(url, json=fake_result, complete_qs=True)
            actual = group.delete(uuid=fake_resource_id)
            assert actual == fake_result

    def test_search(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_search_pattern = "queryString=agentInstalled:true"
        fake_raw_result = ["unit", "test", "search", "values"]
        count = len(fake_raw_result)
        fake_cooked_result = {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": fake_raw_result,
        }
        with requests_mock.Mocker() as m:
            url_suffix = "search?{0}".format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.search(pattern=fake_search_pattern)
            assert actual == fake_cooked_result

    def test_minimal(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_search_pattern = "queryString=agentInstalled:true"
        fake_raw_result = ["unit", "test", "minimal", "result"]
        count = len(fake_raw_result)
        fake_cooked_result = {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": fake_raw_result,
        }
        with requests_mock.Mocker() as m:
            url_suffix = "minimal?{0}".format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.minimal(pattern=fake_search_pattern)
            assert actual == fake_cooked_result

    def test_applications(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_resource_id = "789012"
        fake_raw_result = ["unit", "test", "applications", "result"]
        count = len(fake_raw_result)
        fake_cooked_result = {
            "totalResults": count,
            "pageSize": count,
            "totalPages": 1,
            "pageNo": 1,
            "previousPageNo": 0,
            "nextPage": False,
            "descendingOrder": False,
            "results": fake_raw_result,
        }
        with requests_mock.Mocker() as m:
            url_suffix = "{0}/applications".format(fake_resource_id)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.applications(uuid=fake_resource_id)
            assert actual == fake_cooked_result

    def test_availability(self):
        group = self.client.resources()
        # this a unix epoch timestamp. its actual value is not important, so
        # i just looked at the clock when i first wrote this function.
        fake_start = 1584727225
        fake_end = fake_start + (60 * 24)
        fake_resource_id = "789012"
        fake_result = ["unit", "test", "availability", "result"]
        with requests_mock.Mocker() as m:
            url_suffix = "{0}/availability?startTime={1}&endTime={2}".format(
                fake_resource_id, fake_start, fake_end
            )
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_result, complete_qs=True)
            actual = group.availability(
                uuid=fake_resource_id, start_epoch=fake_start, end_epoch=fake_end
            )
            assert actual == fake_result

    def test_get_templates(self):
        group = self.client.resources()
        fake_resource_id = "789012"
        fake_result = ["unit", "test", "availability", "result"]
        with requests_mock.Mocker() as m:
            url_suffix = "{0}/templates/search?{1}".format(
                fake_resource_id, "fake_pattern"
            )
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_result, complete_qs=True)
            actual = group.get_templates(uuid=fake_resource_id, pattern="fake_pattern")
            assert actual["results"] == fake_result



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# resources.py
# Resource classes.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi
from opsramp.resources import list2ormp


class ResourceGroups(ORapi):
    def __init__(self, parent):
        super(ResourceGroups, self).__init__(parent.api, "deviceGroups")

    def get(self):
        temp_url = "/search"
        return self.api.get(temp_url)

    def create(self, definition):
        url_suffix = ""
        return self.api.post(url_suffix, json=definition)

    def update(self, definition):
        # UUID(s) specified in the request body(ies).
        return self.api.post(json=definition)

    def delete(self, uuid):
        url_suffix = uuid
        return self.api.delete(url_suffix)

    def search(self, pattern=""):
        """returns *verbose* details about resource groups on this tenant"""
        simple_list = super(ResourceGroups, self).search(
            pattern=pattern, suffix="search"
        )
        return list2ormp(simple_list)

    def minimal(self, pattern=""):
        """returns * minimal * details about resource groups on this tenant"""
        simple_list = super(ResourceGroups, self).search(
            pattern=pattern, suffix="minimal"
        )
        return list2ormp(simple_list)



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import unittest

import opsramp.binding
from opsramp.msp import Clients
import requests_mock


class StaticsTest(unittest.TestCase):
    def setUp(self):
        self.hours = {
            "businessStartHour": 10,
            "businessStartMin": 40,
            "businessEndHour": 19,
            "businessEndMin": 23,
            "businessDayStart": 1,
            "businessDayEnd": 4,
            "smsVoiceNotification": True,
        }
        self.client = {
            "name": "unit test client",
            "address": "Springfield",
            "timeZone": "America/Los_Angeles",
            "country": "United States",
        }

    def test_mkHours(self):
        actual = Clients.mkHours(
            day_start=datetime.time(
                self.hours["businessStartHour"], self.hours["businessStartMin"]
            ),
            day_end=datetime.time(
                self.hours["businessEndHour"], self.hours["businessEndMin"]
            ),
            week_start=self.hours["businessDayStart"],
            week_end=self.hours["businessDayEnd"],
            sms_voice_notification=self.hours["smsVoiceNotification"],
        )
        assert actual == self.hours

    def test_mkClient(self):
        # use the default "hours" value first.
        actual = Clients.mkClient(
            name=self.client["name"],
            address=self.client["address"],
            time_zone=self.client["timeZone"],
            country=self.client["country"],
        )
        assert actual == self.client
        # now a specific hours definition
        self.client["clientDetails"] = self.hours
        actual = Clients.mkClient(
            name=self.client["name"],
            address=self.client["address"],
            time_zone=self.client["timeZone"],
            country=self.client["country"],
            hours=self.hours,
        )
        assert actual == self.client
        # remove the hours specifier again, for the next test.
        del self.client["clientDetails"]


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_msp_id = "msp_123456"
        self.msp = self.ormp.tenant(self.fake_msp_id)
        assert not self.msp.is_client()

        self.clients = self.msp.clients()
        assert self.clients

    def test_search(self):
        thisid = 444444
        expected = {"id": thisid}
        url = self.clients.api.compute_url("search?queryString=id:%s" % thisid)
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            actual = self.clients.search("id:%s" % thisid)
            assert actual == expected

    def test_minimal(self):
        thisid = 555555
        expected = {"id": thisid}
        url = self.clients.api.compute_url("minimal")
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            # specific suffix.
            actual = self.clients.get("minimal")
            assert actual == expected
            # default suffix should be the same.
            actual = self.clients.get()
            assert actual == expected

    def test_create_update(self):
        bad_definition = {"bogus": "dude"}
        with requests_mock.Mocker() as m:
            # the name field is missing so we should get an error.
            assert "name" not in bad_definition
            with self.assertRaises(AssertionError):
                actual = self.clients.create(definition=bad_definition)
            assert m.call_count == 0
        # now let's try a valid one.
        good_definition = {
            "name": "elvis",
            "address": "graceland",
            "timeZone": "UTC",
            "country": "US",
        }
        thisid = 555555
        expected_send = good_definition
        expected_receive = {"id": thisid}
        url = self.clients.api.compute_url()
        with requests_mock.Mocker() as m:
            adapter = m.post(url, json=expected_receive, complete_qs=True)
            actual = self.clients.create(definition=good_definition)
            assert adapter.last_request.json() == expected_send
            assert actual == expected_receive
        # now try update
        url = self.clients.api.compute_url(thisid)
        with requests_mock.Mocker() as m:
            adapter = m.post(url, json=expected_receive, complete_qs=True)
            actual = self.clients.update(uuid=thisid, definition=good_definition)
            assert adapter.last_request.json() == expected_send
            assert actual == expected_receive

    def test_suspend(self):
        thisid = 789012
        expected = {"id": thisid}
        url = self.clients.api.compute_url("%s/suspend" % thisid)
        with requests_mock.Mocker() as m:
            m.post(url, json=expected, complete_qs=True)
            actual = self.clients.suspend(uuid=thisid)
            assert actual == expected

    def test_activate(self):
        thisid = 345678
        expected = {"id": thisid}
        url = self.clients.api.compute_url("%s/activate" % thisid)
        with requests_mock.Mocker() as m:
            m.post(url, json=expected, complete_qs=True)
            actual = self.clients.activate(uuid=thisid)
            assert actual == expected

    def test_terminate(self):
        thisid = 901234
        expected = {"id": thisid}
        url = self.clients.api.compute_url("%s/terminate" % thisid)
        with requests_mock.Mocker() as m:
            m.post(url, json=expected, complete_qs=True)
            actual = self.clients.terminate(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class KBtest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.kb = self.client.kb()
        assert "KnowledgeBase" in str(self.kb)


class CategoriesTest(KBtest):
    def setUp(self):
        super(CategoriesTest, self).setUp()
        self.categories = self.kb.categories()
        assert "KBcategories" in str(self.categories)

    def test_create(self):
        group = self.categories
        url = group.api.compute_url("create")
        expected = {"id": 345678}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
        assert actual == expected

    def test_update(self):
        group = self.categories
        thisid = 123456
        url = group.api.compute_url("update/%s" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=expected)
        assert actual == expected

    def test_delete(self):
        group = self.categories
        thisid = 789012
        url = group.api.compute_url("delete/%s" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
        assert actual == expected

    def test_restore(self):
        group = self.categories
        thisid = 345678
        url = group.api.compute_url("restore/%s" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.restore(uuid=thisid)
        assert actual == expected

    def test_get_id(self):
        group = self.categories
        thisid = 345678
        url = group.api.compute_url(thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(thisid)
            assert actual == expected

    def test_children(self):
        group = self.categories
        thisid = 901234
        # searches happen at the "kb" level
        url = self.kb.api.compute_url("categorylist/%s" % thisid)
        expected = ["unit", "test", "list"]
        with requests_mock.Mocker() as m:
            assert expected
            m.get(url, json=expected, complete_qs=True)
            actual = group.children(thisid)
        assert actual == expected

    def test_search(self):
        group = self.categories
        pattern = "whatever"
        # searches happen at the "kb" level
        url = self.kb.api.compute_url("categorylist?%s" % pattern)
        expected = ["unit", "test", "list"]
        with requests_mock.Mocker() as m:
            assert expected
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern=pattern)
        assert actual == expected


class ArticlesTest(KBtest):
    def setUp(self):
        super(ArticlesTest, self).setUp()
        self.articles = self.kb.articles()
        assert "KBarticles" in str(self.articles)

    def test_create(self):
        group = self.articles
        url = group.api.compute_url()
        expected = {"id": 345678}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
        assert actual == expected

    def test_update(self):
        group = self.articles
        thisid = 123456
        url = group.api.compute_url("%s" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=expected)
        assert actual == expected

    def test_delete(self):
        group = self.articles
        thisid = 789012
        url = group.api.compute_url("%s/delete" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
        assert actual == expected

    def test_share(self):
        group = self.articles
        thisid = 789012
        url = group.api.compute_url("%s/share" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.share(uuid=thisid, definition=expected)
        assert actual == expected

    def test_get_id(self):
        group = self.articles
        thisid = 345678
        url = group.api.compute_url(thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(thisid)
            assert actual == expected

    def test_comments(self):
        group = self.articles
        thisid = 567890
        url = group.api.compute_url("%s/comments/" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            actual = group.comments(thisid)
            assert actual == expected

    def test_search(self):
        group = self.articles
        pattern = "whatever"
        # searches happen at the "kb" level
        url = self.kb.api.compute_url("articles/search?%s" % pattern)
        expected = ["unit", "test", "list"]
        with requests_mock.Mocker() as m:
            assert expected
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
        assert actual == expected


class TemplatesTest(KBtest):
    def setUp(self):
        super(TemplatesTest, self).setUp()
        self.templates = self.kb.templates()
        assert "KBtemplates" in str(self.templates)

    def test_create(self):
        group = self.templates
        url = group.api.compute_url()
        expected = {"id": 345678}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=expected)
        assert actual == expected

    def test_update(self):
        group = self.templates
        thisid = 123456
        url = group.api.compute_url("%s" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=expected)
        assert actual == expected

    def test_delete(self):
        group = self.templates
        thisid = 789012
        url = group.api.compute_url("%s/delete" % thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            assert expected
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
        assert actual == expected

    def test_get_id(self):
        group = self.templates
        thisid = 345678
        url = group.api.compute_url(thisid)
        expected = {"id": thisid}
        with requests_mock.Mocker() as m:
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(thisid)
            assert actual == expected

    def test_search(self):
        group = self.templates
        pattern = "whatever"
        # searches happen at the "kb" level
        url = self.kb.api.compute_url("templatesList?%s" % pattern)
        expected = ["unit", "test", "list"]
        with requests_mock.Mocker() as m:
            assert expected
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern=pattern)
        assert actual == expected



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os

import opsramp.binding


def connect():
    url = os.environ["OPSRAMP_URL"]
    key = os.environ["OPSRAMP_KEY"]
    secret = os.environ["OPSRAMP_SECRET"]
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true")
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ["OPSRAMP_TENANT_ID"]

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    tid_list = [(tenant_id, "")]
    if not tenant.is_client():
        children = tenant.clients().get()
        cdata = [(x["uniqueId"], x["name"]) for x in children]
        tid_list.extend(cdata)

    for uuid, name in tid_list:
        child = ormp.tenant(uuid)
        categs = child.rba().categories()
        clist = categs.get()
        print('{0} "{1}"'.format(uuid, name))
        for cdata in clist:
            show_contents(categs, cdata, 1)


def show_contents(categs, cdata, indent):
    print("{0} category {1:05d} {2}".format(".." * indent, cdata["id"], cdata["name"]))
    # get the list of scripts in the category.
    thiscat = categs.category(cdata["id"])
    try:
        scripts = thiscat.get()
    except RuntimeError as e:
        # we have encountered broken trees in the field so guard against it.
        print(e)
        scripts = []
    # print out a short description of each script.
    for s in scripts:
        print(
            '{0}. script {1:06d} {2} {3} "{4}" "{5}"'.format(
                ".." * indent,
                s["id"],
                s["platforms"],
                s["executionType"],
                s["name"],
                s["description"],
            )
        )
    # recurse into any embedded categories.
    if "childs" in cdata:
        for child in cdata["childs"]:
            show_contents(categs, child, indent + 1)


if __name__ == "__main__":
    main()
